namespace BlubTest

open Microsoft.VisualStudio.TestTools.UnitTesting
open Parser
open Evaluator
open AST

[<TestClass>]
type TestClass () =

    [<TestMethod>]
    member this.TestEvalNumAST() =
        let ast = Num 1
        let expected = Num 1
        // My eval method returns a result and an environment map.
        // I don't need the environment here, so I throw it away.
        let actual, _ = eval ast Map.empty
        // In an Assert, the first argument is usually the "expected"
        // value; try to follow this convention so that MsTest gives
        // you informative outputs.
        Assert.AreEqual(expected, actual)

    [<TestMethod>]
    member this.TestEvalAdditionAST() =
        let ast = Plus (Num 1, Num 2)
        let expected = Num 3
        let actual, _ = eval ast Map.empty
        Assert.AreEqual(expected, actual)

    [<TestMethod>]
    member this.TestParsedAddition() =
        let program = "1 + 1"
        let expected = Num 2
        let ast_maybe = parse program false
        match ast_maybe with
        | Some ast ->
            let actual, _ = eval ast Map.empty
            Assert.AreEqual(expected, actual)
        | None ->
            Assert.IsTrue(false)

    [<TestMethod>]
    member this.TestParsedAssignment() =
        let program = "x := 1 + 2"
        // the expected return value
        let expected = Num 3
        // the expected environment
        let expected_env = [("x", Num 3)] |> Map.ofList
        let ast_maybe = parse program false
        match ast_maybe with
        | Some ast ->
            let actual, actual_env = eval ast Map.empty
            // check the return value
            Assert.AreEqual(expected, actual)
            // check the environment
            Assert.AreEqual(expected_env, actual_env)
        | None ->
            Assert.IsTrue(false)
